# pio_uart_tx_parity_V2.py - 12.09.2021
#
# uart Transmitter, mit Parity-bit, mit 1 /2 Stoppbits
#
# Parity-Vit:
# Am Beginn eines Bytes wird das X-Register mit einem graden Wert (für Even-Parity)
# bzw. einem ungeraden Wert (für Odd-Parity) vorbelegt (der Wert muss glrßer als 8 sein).
# Nach dem Schreiben eines Bits wird dieses per jmp(pin) getestet.
# Der JMP-Pin ist ein zusätzlicher Pin, der als Eingang parallel zum TX-Pin geschaltet
# werden muss.
# Ist das Bit gesetzt, dann wird das X-Register decrementiert,
# ist das Bit micht gesetzt. wird X nicht verändert.
# Am Ende wird das niedrigstwertige Bit des X-Registers als Parity-Bit geschrieben.
#
# Vorteil: benöitgt nur 11 Instruktionen
# NAchtei: benötigt einen zusätzlichen Pin 

from machine import Pin
from rp2 import PIO, StateMachine, asm_pio
import time

@asm_pio(out_init=PIO.OUT_HIGH,
         set_init=PIO.OUT_HIGH,
         fifo_join=rp2.PIO.JOIN_TX,
         out_shiftdir=PIO.SHIFT_RIGHT,
         pull_thresh=8)

def my_uart_tx():

    wrap_target()
    pull()                     #  1 Warten auf Daten 
                               #    X-Register nutzen für Paritätaberechnung
                               #    X darf nicht zu 0 deccrementiert werden,
                               #    daher muss X > 8 sein,
    # set(x, 16)               #  2 Geradzahliges  X -> Paritätsbit = 1 bei Even Parity
    set(x, 15)                 #  2 Ungradzahliges X -> Paritätsbit = 1 bei Odd  Parity
    
                               #    Jetzt sind Daten eingegangen ...
    set(pins, 0)           [8] #  3 Das Startbit setzen für 9 Takte ( 1 + 8 = 9 Takte)
    
    label("next_bit")
    out(pins, 1)           [5] #  4 Das nächste Bit ausgeben
    
    jmp(pin, "dec")            #  5 Der PIn ist gesetzt, Parity decrementieren
    jmp("no_dec")              #  6 der Pin ist low, nichts tun
    
    label("dec")
    jmp(x_dec, "no_dec")       #  7 wenn der Oin high ist, dann x decrementieren
                               #    x == 0 tritt nie ein, da x groß genug gewählt 
    label("no_dec")
    jmp(not_osre, "next_bit")  #  8 nächstes Bit ausgeben, solange das OSR Daten enthält
                               #    Alle 8 Bits sind rausgeschiftet
    mov(osr, x)                #  9 x nach OSR kopiern
    out(pins, 1)           [8] # 10 und das niedrigstwertige Bit als Parity-Bit ausgeben
                               #    1 Stopbit senden
    set(pins, 1)           [6] # 11 Das Stopbit für 9 Takt setzen (1 + 6 + pull + set)
                               #    alternativ: 2 Stopbits senden
    #set(pins, 1)         [15] # 11 Das Stopbit für 18 Takt setzen (1 + 15 + pull + set)
    wrap()

# --- Einsetllungen ----------------------------

tx_pin = 16
BAUD = 9600

# ----------------------------------------------

sm = StateMachine(4, my_uart_tx, freq=9*BAUD,
                  out_base=Pin(tx_pin),
                  set_base=Pin(tx_pin),
                  jmp_pin=Pin(tx_pin),
                  in_base=Pin(tx_pin + 1,  Pin.IN) )
sm.restart()
sm.active(1)

time.sleep(.1)
for i in range(32):
    sm.put(65 + i)


print("\ndone")
