# pio_uart_tx_parity_soft.py - 12.09.2021
#
# UART Transmitter mit Paritätsbit,
# das Parity-Bit wird per Software berechnet und es 
# werden 8 Datenbits + 1 Paritätsbit an die Statemachine übertragen !
# Optional mit 2 Stopbits.

from machine import Pin
from rp2 import PIO, StateMachine, asm_pio
import time

@asm_pio(out_init=PIO.OUT_HIGH,
         set_init=PIO.OUT_HIGH,
         fifo_join=rp2.PIO.JOIN_TX,
         out_shiftdir=PIO.SHIFT_RIGHT,
         pull_thresh=9)

def my_uart_tx():

    wrap_target()
    pull()                     #  1 Warten auf Daten in der Fifo
                               #    Jetzt sind Daten eingegangen ...
    set(pins, 0)           [8] #  2 Das Startbit setzen für 9 Takte
    
    label("next_bit")
    out(pins, 1)           [7] #  3 Das nächste Bit ausgeben

    jmp(not_osre, "next_bit")  #  4 nächstes Bit ausgeben, solange OSR Daten enthält
                               #    Alle 8 Bits sind rausgeschiftet
                               #    1 Stopbit senden
    set(pins, 1)           [7] #  5 Das Stopbit für 9 Takt setzen (1 + 7 + pull )
                               #    alternativ: 2 Stopbits senden
    #set(pins, 1)         [16] #  5 Das Stopbit für 18 Takt setzen (1 + 16 + pull)

    wrap()

# --- Einstellungen ----------------------

tx_pin = 16
BAUD = 9600

# ------------------------------------------

sm = StateMachine(4, my_uart_tx, freq=9* BAUD,
                  out_base=Pin(tx_pin),
                  set_base=Pin(tx_pin) )
sm.restart()
sm.active(1)

def send_with_parity(data):
    # Das Paritätsbit hinzugügen
    bits = data 
    p = 1         # Startwert: 0 für even, 1 für odd
    for i in range(8):
        p += bits & 0x01
        bits //= 2
    sm.put((p & 0x01) * 0x100 | data)

time.sleep(.1)
for i in range(4):
    data = 0x55 + i
    send_with_parity(data)

time.sleep(.6)

print("\ndone")
